#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=LatticeSym
#pragma version = 2.0
//
//	remember to execute    InitLatticeSymPackage()
//
//	with version 2.0, added a text description to the crystalLattice structure


Menu "Analysis"
	SubMenu "Lattice"
//		"Set Crystal Lattice...",setLattice()
		"Set Crystal Lattice...",MakeLatticeParametersPanel("")
		help={"Define the crystal structure and lattice to be used"}
		"Show Crystal Lattice",showLattice()
		help={"Shows the crystal structure and lattice that is currently defined"}
		"d[hkl]",/Q,get_dhkl(NaN,NaN,NaN)
		help={"show d-spacing for the hkl reflection"}
		"F[structure, hkl)]",getFstruct(NaN,NaN,NaN,NaN)
		help={"Crude Structure Factor using current lattice"}
	End
End

Strconstant PresetLatticeStructures = "FCC:225;BCC:229;Diamond:227;Simple Cubic:221;Hexagonal:194;Wurtzite (B4):186;Sapphire:167;Triclinic:1;other"
//OverRide Strconstant PresetLatticeStructures = "\"FCC:225;BCC:229;Diamond:227;Simple Cubic:221;Hexagonal:194;Wurtzite (B4):186;Sapphire:167;Triclinic:1;other\""


// =========================================================================
// =========================================================================
//	Start of setting particular lattice constants

Function MakeLatticeParametersPanel(strStruct)
	String strStruct									// optional passed value of geo structure, this is used if passed
	if (strlen(WinList("LatticeSet",";","WIN:64")))	// window alreay exits, bring it to front
		DoWindow/F LatticeSet
		return 0
	endif
	NewPanel /K=1 /W=(675,60,895,425)
	DoWindow/C LatticeSet
	FillLatticeParametersPanel(strStruct,"LatticeSet",0,0)
End

// prompts the user of lattice info, and set local structure.  Store the information in a structure string in current datafolder
Function setLattice()
	String strName=""
	Variable structNum
	Variable a,b,c,alpha,bet,gam
	String desc

	STRUCT crystalLattice xtal
	xtal.a = 0
	FillLatticeStructDefault(xtal)						//fill the lattice structure with default values
	a = xtal.a			;	b = xtal.b		;	c = xtal.c
	alpha = xtal.alpha	;	bet = xtal.beta	;	gam = xtal.gam
	structNum = xtal.structNum
	desc = xtal.desc

	if (GetLatticeConstants(xtal,structNum,strName,a,b,c,alpha,bet,gam,desc))
		return 1
	endif
	if (exists(":xtalStructStr")==2)					// use the global in this data folder
		SVAR xtalStructStr=:xtalStructStr
	elseif (exists("root:Packages:Lattices:xtalStructStr")==2)	// use the global in the Packages
		SVAR xtalStructStr=root:Packages:Lattices:xtalStructStr
	else
		String/G xtalStructStr							// no global, make one in this data folder
	endif
	StructPut/S/B=2 xtal, xtalStructStr				// and write them to the global
	String sym = getSymString(xtal.structNum)		// symmetry symbol
	printf "changed lattice to  '%s'  #%d   %s     %.9gnm, %.9gnm, %.9gnm,   %g, %g, %g\r",xtal.desc,xtal.structNum,sym,xtal.a,xtal.b,xtal.c,xtal.alpha,xtal.beta,xtal.gam
	SavePackagePreferences/FLSH=1 "LatticeSym","LatticeSymPrefs",0,xtal
	return 0
End


Function showLattice()									// prints the structure that is currently being used
	STRUCT crystalLattice xtal
	if (FillLatticeStructDefault(xtal))					//fill the lattice structure with test values
		DoAlert 0, "no lattice structure found"
		return 1
	endif
	String str, sym = getSymString(xtal.structNum)
	sprintf str, "current lattice is  #%d   %s\r%.9g, %.9g, %.9gnm,\r%g, %g, %g",xtal.structNum,sym,xtal.a,xtal.b,xtal.c,xtal.alpha,xtal.beta,xtal.gam
	DoAlert 1,str+"\r\tprint all details to history too?"
	if (V_flag==1)
		print_crystalLattice(xtal)						// prints out the value in a crystalLattice structure
	else
		printf "current lattice is  #%d   %s     %.9gnm, %.9gnm, %.9gnm,   %g, %g, %g\r",xtal.structNum,sym,xtal.a,xtal.b,xtal.c,xtal.alpha,xtal.beta,xtal.gam
		if (!(xtal.alphaT<=0))
			printf "coefficient of thermal expansion is %g\r",xtal.alphaT
		endif
	endif
	return 0
End


Structure crystalLattice		// structure definition for a crystal lattice
	double a,b,c				// latticde constants, length (nm)
	double alpha,beta,gam		// angles (degree)
	int16 structNum			// structure number from international tables, allowed range is [1, 230]
	double  a0,  b0,  c0		// direct lattice from constants { a[], b[], c[] }
	double  a1,  b1,  c1
	double  a2,  b2,  c2
	double  as0,  bs0,  cs0	// reciprocal lattice { a*[], b*[], c*[] }
	double  as1,  bs1,  cs1	// a*,b*,c* already have the 2PI in them
	double  as2,  bs2,  cs2
	double Vc					// volume of cell, triple product of (a[]xb[]).c
	double	alphaT				// coef of thermal expansion, a = ao*(1+alphaT*(TempC-22.5))
	char desc[100]			// name or decription of this lattice
EndStructure
//
Function print_crystalLattice(xtal)			// prints out the value in a crystalLattice structure
	STRUCT crystalLattice &xtal				// this sruct is set in this routine
	printf "lattice constants  { %.8fnm, %.8fnm, %.8fnm,   %g, %g, %g }\r",xtal.a,xtal.b,xtal.c,xtal.alpha,xtal.beta,xtal.gam
	if (strlen(xtal.desc))
		printf "\t'%s'   \t",xtal.desc
	else
		printf "\t\t\t"
	endif
	printf "stucture no. = %d       %s       Vc = %g (nm^3)",xtal.structNum, getSymString(xtal.structNum),xtal.Vc
	if (!(xtal.alphaT<=0))
		printf "     coefficient of thermal expansion is %g\r",xtal.alphaT
	else
		printf "\r"
	endif
	printf "				a				b				c   (nm)\r"
	printf "direct		%+10.7f\t%+10.7f\t%+10.7f\r",xtal.a0,xtal.b0,xtal.c0
	printf "lattice		%+10.7f\t%+10.7f\t%+10.7f\r",xtal.a1,xtal.b1,xtal.c1
	printf "			%+10.7f\t%+10.7f\t%+10.7f\r",xtal.a2,xtal.b2,xtal.c2
	printf "\r				a*				b*				c*  (1/nm)\r"
	printf "recip	\t%+10.6f\t%+10.6f\t%+10.6f\r",xtal.as0,xtal.bs0,xtal.cs0
	printf "lattice	\t%+10.6f\t%+10.6f\t%+10.6f\r",xtal.as1,xtal.bs1,xtal.cs1
	printf "		\t%+10.6f\t%+10.6f\t%+10.6f\r",xtal.as2,xtal.bs2,xtal.cs2
End


// gets lattice constants for known structures via dialogs,  it uses structure numbers from the International Tables
// known structures are:
//	FCC			225
//	BCC			229
//	Diamond		227
//	Simple Cubic	221
//	Hexagonal		194
//	Sapphire		167
//	Triclinic		1
Static Function GetLatticeConstants(xtal,structureNum,structureName,a,b,c,alpha,bet,gam,desc)
	STRUCT crystalLattice &xtal							// this sruct is set in this routine
	Variable structureNum								// same as for Internationl tables
	String structureName									// see structures below for list of valid names
	Variable a,b,c,alpha,bet,gam							// lattice constants, lengths in nm angles in degrees
	String desc											// description of lattice (i.e. the name)
	a = numtype(a)||a<=0 ? 0.405 : a					// default is aluminum
	b = numtype(b)||b<=0 ? a : b
	c = numtype(c)||c<=0 ? a : c
	alpha = numtype(alpha)||alpha<=0 ? 90 : alpha	// for invalid angles, use 90
	bet = numtype(bet)||bet<=0 ? 90 : bet
	gam = numtype(gam)||gam<=0 ? 90 : gam
	//	Cubic			[195,230]	//	a
	//	Hexagonal		[168,194]	//	a,c
	//	Trigonal		[143,167]	//	a,alpha
	//	Tetragonal		[75,142]		//	a,c
	//	Orthorhombic	[16,74]		//	a,b,c
	//	Monoclinic		[3,15]			//	a,b,c,gamma
	//	Triclinic		[1,2]			//	a,b,c,alpha,beta,gamma
	String item="",structures="FCC:225;BCC:229;Diamond:227;Simple Cubic:221;Hexagonal:194;Wurtzite (B4):186;Sapphire:167;Triclinic:1;other"
	Variable i,N=ItemsInList(structures)
	if (structureNum>=1 && structureNum<=230)	// valid structureNum
		structureName = ""
		for(i=0;i<N;i+=1)
			item = StringFromList(i,structures)
			if (structureNum==str2num(StringFromList(1,item,":")))
				structureName = StringFromList(0,item,":")
			endif
		endfor
		if (strlen(structureName)<1)
			DoAlert 0, "unknown structure number "+num2str(structureNum)
		endif
	endif
	if (strlen(structureName)>1)						// a structureName passed
		structureNum = -1
		for(i=0;i<N;i+=1)
			item = StringFromList(i,structures)
			if (stringmatch(structureName,StringFromList(0,item,":")))
				structureNum = str2num(StringFromList(1,item,":"))
			endif
		endfor
		if (structureNum<1)
			DoAlert 0, "unknown structure name '"+structureName+"'"
		endif
	endif
	item = structureName+":"+num2istr(structureNum)
	Prompt item, "lattice structure", popup, structures
	DoPrompt/HELP="" "lattice structure",item
	if (V_flag)
		return 1
	endif
	if (stringmatch(item,"other"))
		Prompt structureNum,"structure number from International Tables [1,230]"
		DoPrompt "Structure Number",structureNum
		if (V_flag)
			return 1
		endif
	else
		structureNum=str2num(StringFromList(1,item,":"))
		structureName = StringFromList(0,item,":")
	endif
	if (!(1<=structureNum && structureNum<=230))
		return 1
	endif

	Prompt a, "a (nm)"
	Prompt b, "b (nm)"
	Prompt c, "c (nm)"
	Prompt alpha, "alpha ()"
	Prompt bet, "beta ()"
	Prompt gam, "gamma ()"

	if (structureNum>=195)			// Cubic
		DoPrompt/HELP="" "Cubic lattice constants",a
		b=a;  c=a
		alpha=90;  bet=90;  gam=90
	elseif(structureNum>=168)		// Hexagonal
		c = (a==c) ? 3*a : c
		DoPrompt/HELP="" "Hexagonal lattice constants",a,c
		b = a
		alpha=90;  bet=90;  gam=120
	elseif(structureNum>=143)		// Trigonal, use the hexagonal cell
		c = (a==c) ? 3*a : c
		DoPrompt/HELP="" "Trigonal lattice constants",a,c
		b = a
		alpha=90;  bet=90;  gam=120
	elseif(structureNum>=75)		// Tetragonal
		DoPrompt/HELP="" "Tetragonal lattice constants",a,c
		b = a
		alpha=90;  bet=90;  gam=90
	elseif(structureNum>=16)		// Orthorhombic
		DoPrompt/HELP="" "Orthorhombic lattice constants",a,b,c
		alpha=90;  bet=90;  gam=90
	elseif(structureNum>=3)		// Monoclinic
		DoPrompt/HELP="" "Monoclinic lattice constants",a,b,c,gam
		alpha=90;  bet=90
	else								// Triclinic
		DoPrompt/HELP="" "Triclinic lattice constants",a,b,c,alpha,bet,gam
	endif

	xtal.a = a			;	xtal.b = b		;	xtal.c = c	// put values into structure
	xtal.alpha = alpha	;	xtal.beta = bet	;	xtal.gam = gam
	xtal.structNum=structureNum
	ForceLatticeToStructure(xtal)
	return 0
End
//
// forces lattice constants to match the structure number (e.g. for cubic, forces b and c to be a, and all angles 90)
Static Function ForceLatticeToStructure(xtal)
	STRUCT crystalLattice &xtal							// this sruct is set in this routine

	Variable structureNum=xtal.structNum				// local value for convienence
	if (structureNum!=limit(structureNum,1,230))	// invalid structureNum, it must be in range [1,230]
		DoAlert 0, "invalid structure number "+num2str(structureNum)
		return 1
	endif
	//	Cubic			[195,230]	//	a
	//	Hexagonal		[168,194]	//	a,c
	//	Trigonal		[143,167]	//	a,alpha
	//	Tetragonal		[75,142]		//	a,c
	//	Orthorhombic	[16,74]		//	a,b,c
	//	Monoclinic		[3,15]			//	a,b,c,gamma
	//	Triclinic		[1,2]			//	a,b,c,alpha,beta,gamma

	if (structureNum>=195)		// Cubic
		xtal.b = xtal.a
		xtal.c = xtal.a
		xtal.alpha=90;  xtal.beta=90;  xtal.gam=90
	elseif(structureNum>=168)	// Hexagonal
		xtal.b = xtal.a
		xtal.alpha=90;  xtal.beta=90;  xtal.gam=120
	elseif(structureNum>=143)	// Trigonal, use the hexagonal cell
		xtal.b = xtal.a
		xtal.alpha=90;  xtal.beta=90;  xtal.gam=120
	elseif(structureNum>=75)		// Tetragonal
		xtal.b = xtal.a
		xtal.alpha=90;  xtal.beta=90;  xtal.gam=90
	elseif(structureNum>=16)		// Orthorhombic
		xtal.alpha=90;  xtal.beta=90;  xtal.gam=90
	elseif(structureNum>=3)		// Monoclinic
		xtal.alpha=90;  xtal.beta=90
//	else								// Triclinic
	endif

	String str
	if (xtal.a<=0 || xtal.b<=0 || xtal.c<=0 || numtype(xtal.a+xtal.b+xtal.c))	// check for valid a,b,c
		sprintf str,"invalid, (a,b,c) = (%g,%g,%g)",xtal.a,xtal.b,xtal.c
		DoAlert 0, str
		return 1
	endif
	Variable alpha=xtal.alpha,bet=xtal.beta,gam=xtal.gam
	if (alpha<=0 || bet<=0 || gam<=0 || alpha>=180 || bet>=180 || gam>=180 || numtype(alpha+bet+gam))	// check for valid angles
		sprintf str,"invalid, (alpha,beta,gam) = (%g,%g,%g)",alpha,bet,gam
		DoAlert 0, str
		return 1
	endif
	setDirectRecip(xtal)								// update Vc, direct and recip
	return 0
End
//
Static Function setDirectRecip(xtal)					// set direct and recip lattice vectors from a,b,c,...
	STRUCT crystalLattice &xtal

	Variable a=xtal.a, b=xtal.b, c=xtal.c
	Variable sa = sin((xtal.alpha)*PI/180), ca = cos((xtal.alpha)*PI/180)
	Variable cb = cos((xtal.beta)*PI/180), cg = cos((xtal.gam)*PI/180)
	Variable phi = sqrt(1.0 - ca*ca - cb*cb - cg*cg + 2*ca*cb*cg)	// = Vc/(a*b*c)
	xtal.Vc = a*b*c * phi								// volume of unit cell
	Variable pv = (2*PI) / (xtal.Vc)				// used for scaling
	Variable a0,a1,a2,  b0,b1,b2,  c0,c1,c2		//components of the direct lattice vectors
	a0=a*phi/sa	; a1=a*(cg-ca*cb)/sa	; a2=a*cb
	b0=0			; b1=b*sa				; b2=b*ca
	c0=0			; c1=0					; c2=c
	xtal.a0=a0		; xtal.a1=a1	; xtal.a2=a2
	xtal.b0=b0		; xtal.b1=b1	; xtal.b2=b2
	xtal.c0=c0		; xtal.c1=c1	; xtal.c2=c2
	xtal.as0=(b1*c2-b2*c1)*pv	; xtal.as1=(b2*c0-b0*c2)*pv	; xtal.as2=(b0*c1-b1*c0)*pv	// (b x c)*2/Vc
	xtal.bs0=(c1*a2-c2*a1)*pv	; xtal.bs1=(c2*a0-c0*a2)*pv	; xtal.bs2=(c0*a1-c1*a0)*pv	// (c x a)*2/Vc
	xtal.cs0=(a1*b2-a2*b1)*pv	; xtal.cs1=(a2*b0-a0*b2)*pv	; xtal.cs2=(a0*b1-a1*b0)*pv	// (a x b)*2/Vc
	return 0
End
//Static Function setDirectRecip(xtal)							// set direct and recip lattice vectors from a,b,c,...
//	STRUCT crystalLattice &xtal
//
//	Variable a=xtal.a, b=xtal.b, c=xtal.c
//	Variable alpha=xtal.alpha, bet=xtal.beta, gam=xtal.gam
//	alpha *= PI/180									// convert internal values to rad
//	bet *= PI/180
//	gam *= PI/180
//
//	Variable a0,a1,a2,  b0,b1,b2,  c0,c1,c2				//components of the direct lattice vectors
//	a0=a			; a1=0				; a2=0			// a vec is along x-axis
//	b0=b*cos(gam)	; b1=b*sin(gam)	; b2=0			// b vec lies in x-y plane, rotated from x toward y
//	c0=c*cos(bet)
//	c1 = c*(cos(alpha)-cos(bet)*cos(gam)) / sin(gam)
//	c2 = sqrt(c*c - c0*c0 - c1*c1)					// c vec projects up into +z direction
//	xtal.a0=a0		; xtal.a1=a1		; xtal.a2=a2
//	xtal.b0=b0		; xtal.b1=b1		; xtal.b2=b2
//	xtal.c0=c0		; xtal.c1=c1		; xtal.c2=c2
//
//	Variable x0,x1,x2									// components of a cross product, used to get a*,b*,c*
//	x0 = b1*c2-b2*c1  ;  x1 = b2*c0-b0*c2  ;  x2 = b0*c1-b1*c0	// (b x c)
//	Variable Vc = a0*x0 + a1*x1 + a2*x2				// Volume of unit cell, the triple product
//	xtal.Vc = Vc
//
//	Variable as0,as1,as2,  bs0,bs1,bs2,  cs0,cs1,cs2	//components of the recip lattice vectors
//	Vc /= 2*PI											// put the 2*PI in here, this is like multiplying a* x 2pi
//	as0 = x0/Vc	;  as1 = x1/Vc	;  as2 = x2/Vc		// (b x c)/Vc
//	bs0 = (c1*a2-c2*a1)/Vc  ;  bs1 = (c2*a0-c0*a2)/Vc  ;  bs2 = (c0*a1-c1*a0)/Vc		// (c x a)/Vc
//	cs0 = (a1*b2-a2*b1)/Vc  ;  cs1 = (a2*b0-a0*b2)/Vc  ;  cs2 = (a0*b1-a1*b0)/Vc	// (a x b)/Vc
//	xtal.as0=as0		; xtal.as1=as1		; xtal.as2=as2
//	xtal.bs0=bs0		; xtal.bs1=bs1		; xtal.bs2=bs2
//	xtal.cs0=cs0		; xtal.cs1=cs1		; xtal.cs2=cs2
//End


Function get_dhkl(h,k,l)
	Variable h,k,l
	STRUCT crystalLattice xtal
	String xtalStructString=StrVarOrDefault(":xtalStructStr","")	// set to values in current directory
	if (FillLatticeStructDefault(xtal))					//fill the lattice structure with test values
		DoAlert 0, "no lattice structure found"
		return NaN
	endif
	if (numtype(h+k+l))
		h = numtype(h) ? 0 : h
		k = numtype(k) ? 0 : k
		l = numtype(l) ? 2 : l
		Prompt h,"H"
		Prompt k,"K"
		Prompt l,"L"
		DoPrompt "(hkl)",h,k,l
		if (V_flag)
			return NaN
		endif
	endif
	if (numtype(h+k+l))
		return NaN
	endif
	Variable d = dSpacing(xtal,h,k,l)
	if (ItemsInList(GetRTStackInfo(0))<2)
		Variable places = placesOfPrecision(xtal.a)
		printf "d[#%d,  (%s)] = %.9g nm\r",xtal.structNum,hkl2str(h,k,l),roundSignificant(d,places)
	endif
	return d
End
//
Function dSpacing(xtal,h,k,l)			// returns d-spacing for the hkl (nm)
	STRUCT crystalLattice &xtal			// this sruct is set in this routine
	Variable h,k,l
	Variable xx,yy,zz
	xx = h*xtal.as0 + k*xtal.bs0 + l*xtal.cs0
	yy = h*xtal.as1 + k*xtal.bs1 + l*xtal.cs1
	zz = h*xtal.as2 + k*xtal.bs2 + l*xtal.cs2
	return 2*PI/sqrt(xx*xx + yy*yy + zz*zz)
End


Function FillLatticeStructDefault(xtal)				//fill the lattice structure with test values
	STRUCT crystalLattice &xtal						// returns 0 if something set, 0 is nothing done

	String strStruct=StrVarOrDefault(":xtalStructStr","")// set to values in current directory
	if (strlen(strStruct)<1)
//		strStruct=StrVarOrDefault("root:Packages:geometry:xtalStructStr","")	// try the default values
		strStruct=StrVarOrDefault("root:Packages:Lattices:xtalStructStr","")	// try the default values
	endif
	if (strlen(strStruct)>1)
		StructGet/S/B=2 xtal, strStruct				// found structure information, load into xtal
	else
		LoadPackagePreferences/MIS=1 "LatticeSym","LatticeSymPrefs",0,xtal
		if (V_flag)
			return 1									// did nothing, nothing found
		endif
	endif
	setDirectRecip(xtal)
	return 0
End

//	End of setting particular lattice constants
// =========================================================================
// =========================================================================


// =========================================================================
// =========================================================================
//	Start of lattice set panel

Function/T FillLatticeParametersPanel(strStruct,hostWin,left,top)
	String strStruct									// optional passed value of xtal structure, this is used if passed
	String hostWin									// name of home window
	Variable left, top									// offsets from the left and top

	NewDataFolder/O root:Packages:Lattices:PanelValues	// ensure that the needed data folders exist
	Variable new = !(NumVarOrDefault("root:Packages:Lattices:PanelValues:structureNum",-1)>0)
	String/G root:Packages:Lattices:PanelValues:desc
	Variable/G root:Packages:Lattices:PanelValues:structureNum
	Variable/G root:Packages:Lattices:PanelValues:a
	Variable/G root:Packages:Lattices:PanelValues:b
	Variable/G root:Packages:Lattices:PanelValues:c
	Variable/G root:Packages:Lattices:PanelValues:alpha
	Variable/G root:Packages:Lattices:PanelValues:beta
	Variable/G root:Packages:Lattices:PanelValues:gam
	Variable/G root:Packages:Lattices:PanelValues:alphaT
	Variable/G root:Packages:Lattices:PanelValues:dirty
	NVAR structureNum=root:Packages:Lattices:PanelValues:structureNum
	SVAR desc=root:Packages:Lattices:PanelValues:desc
	NVAR a=root:Packages:Lattices:PanelValues:a
	NVAR b=root:Packages:Lattices:PanelValues:b
	NVAR c=root:Packages:Lattices:PanelValues:c
	NVAR alpha=root:Packages:Lattices:PanelValues:alpha
	NVAR beta=root:Packages:Lattices:PanelValues:beta
	NVAR gam=root:Packages:Lattices:PanelValues:gam
	NVAR alphaT=root:Packages:Lattices:PanelValues:alphaT
	NVAR dirty=root:Packages:Lattices:PanelValues:dirty

	Variable/G root:Packages:Lattices:PanelValues:h
	Variable/G root:Packages:Lattices:PanelValues:k
	Variable/G root:Packages:Lattices:PanelValues:l
	Variable/G root:Packages:Lattices:PanelValues:dspace_nm
	Variable/G root:Packages:Lattices:PanelValues:Fr
	Variable/G root:Packages:Lattices:PanelValues:Fi

	STRUCT crystalLattice xtal
	if (strlen(strStruct))								// start using the passed values
		StructGet/S/B=2 xtal, strStruct					// found passed structure information, load into xtal
		dirty = 1
	elseif(new)											// no old values present, use usual defaults
		FillLatticeStructDefault(xtal)
		a=xtal.a  ;  b=xtal.b  ;  c=xtal.c
		alpha=xtal.alpha  ;  beta=xtal.beta  ;  gam=xtal.gam
		structureNum=xtal.structNum
		alphaT=xtal.alphaT
		desc=xtal.desc
		dirty = 0
	endif

	SetWindow kwTopWin,userdata(LatticePanelName)=hostWin+"#LatticePanel"
	NewPanel/K=1/W=(left,top,left+221,top+465)/HOST=$hostWin
	ModifyPanel frameStyle=0, frameInset=0
	RenameWindow #,LatticePanel
	SetVariable setDesc,pos={4,13},size={211,18},title="name"
	SetVariable setDesc,help={"description of this lattice"},fSize=12,proc=LatticePanelParamProc
	SetVariable setDesc,limits={-inf,inf,0},value= root:Packages:Lattices:PanelValues:desc
	PopupMenu popLatticeStructure,pos={44,41},size={116,20},proc=LatticePanelPopMenuProc
	PopupMenu popLatticeStructure,font="Baskerville Semibold",fSize=12,fStyle=1
//	PopupMenu popLatticeStructure,mode=5,popvalue="Hexagonal:194",value= #PresetLatticeStructures
//	PopupMenu popLatticeStructure,mode=5,value= #PresetLatticeStructures
	PopupMenu popLatticeStructure,mode=5,value= #"PresetLatticeStructures"
	SetVariable set_structNum,pos={45,67},size={110,18},proc=LatticePanelParamProc,title="structure"
	SetVariable set_structNum,fSize=12,format="%d",userdata(oldValue)=num2istr(structureNum)
	SetVariable set_structNum,limits={1,230,0},value= root:Packages:Lattices:PanelValues:structureNum
	TitleBox structureTitle,pos={25,95},size={82,15},title="\\JC"
	TitleBox structureTitle,fSize=16,frame=0,anchor= LC
	SetVariable set_a_nm,pos={45,125},size={128,18},proc=LatticePanelParamProc,title="a (nm)"
	SetVariable set_a_nm,fSize=12,format="%.7f"
	SetVariable set_a_nm,limits={0,inf,0},value= root:Packages:Lattices:PanelValues:a
	SetVariable set_b_nm,pos={45,145},size={128,18},proc=LatticePanelParamProc,title="b (nm)"
	SetVariable set_b_nm,fSize=12,format="%.7f"
	SetVariable set_b_nm,limits={0,inf,0},value= root:Packages:Lattices:PanelValues:b
	SetVariable set_c_nm,pos={45,165},size={128,18},proc=LatticePanelParamProc,title="c (nm)"
	SetVariable set_c_nm,fSize=12,format="%.7f"
	SetVariable set_c_nm,limits={0,inf,0},value= root:Packages:Lattices:PanelValues:c
	SetVariable set_alpha,pos={45,195},size={100,17},proc=LatticePanelParamProc,title="a"
	SetVariable set_alpha,font="Symbol",fSize=14,format="%.5f"
	SetVariable set_alpha,limits={0,180,0},value= root:Packages:Lattices:PanelValues:alpha
	SetVariable set_beta,pos={45,215},size={100,17},proc=LatticePanelParamProc,title="b"
	SetVariable set_beta,font="Symbol",fSize=14,format="%.5f"
	SetVariable set_beta,limits={0,180,0},value= root:Packages:Lattices:PanelValues:beta
	SetVariable set_gamma,pos={45,235},size={100,17},proc=LatticePanelParamProc,title="g"
	SetVariable set_gamma,font="Symbol",fSize=14,format="%.5f"
	SetVariable set_gamma,limits={0,180,0},value= root:Packages:Lattices:PanelValues:gam
	Button buttonLatticeSave,pos={35,265},size={150,20},proc=LatticePanelButtonProc,title="Save"
	Button buttonLatticeSave,help={"Save these values as the current values"}
	Button buttonLatticeRevert,pos={35,290},size={150,20},proc=LatticePanelButtonProc,title="Revert"
	Button buttonLatticeRevert,help={"revert to the current default"}
	Button buttonLatticeFromFile,pos={35,315},size={150,20},proc=LatticePanelButtonProc,title="Load from a file"
	Button buttonLatticeFromFile,help={"Fill the values in this panel from a file"}
	Button buttonPrintLattice,pos={35,340},size={150,20},proc=LatticePanelButtonProc,title="Print Lattice to History"
	Button buttonPrintLattice,help={"print lattice values to the history"}

	SetVariable h_LatticeVar,pos={12,391},size={60,15},proc=LatticePanelParamProc,title="H",font="Lucida Grande",fSize=12
	SetVariable h_LatticeVar,value= root:Packages:Lattices:PanelValues:h
	SetVariable k_LatticeVar,pos={84,391},size={60,15},proc=LatticePanelParamProc,title="K",font="Lucida Grande",fSize=12
	SetVariable k_LatticeVar,value= root:Packages:Lattices:PanelValues:k
	SetVariable L_LatticeVar,pos={156,391},size={60,15},proc=LatticePanelParamProc,title="L",font="Lucida Grande",fSize=12
	SetVariable L_LatticeVar,value= root:Packages:Lattices:PanelValues:l
	ValDisplay d_LatticeDisp,pos={57,412},size={123,17},title="d (nm)=",font="Lucida Grande",fSize=12,format="%.7f"
	ValDisplay d_LatticeDisp,limits={0,0,0},barmisc={0,1000},value=#"root:Packages:Lattices:PanelValues:dspace_nm"
	ValDisplay d_LatticeDisp,help={"d-spacing (nm) calculated using the lattice"},frame=0
	ValDisplay Fr_3atticeDisp,pos={20,435},size={80,17},title="F =",value= #"root:Packages:Lattices:PanelValues:Fr"
	ValDisplay Fr_3atticeDisp,font="Lucida Grande",fSize=12,format="%.3f",limits={0,0,0},barmisc={0,1000}
	ValDisplay Fr_3atticeDisp,help={"d-spacing (nm) calculated using the lattice"},frame=0
	ValDisplay Fi_3atticeDisp,pos={104,435},size={80,17},title="+i",value= #"root:Packages:Lattices:PanelValues:Fi"
	ValDisplay Fi_3atticeDisp,font="Lucida Grande",fSize=12,format="%.3f",limits={0,0,0},barmisc={0,1000}
	ValDisplay Fi_3atticeDisp,help={"d-spacing (nm) calculated using the lattice"},frame=0

	LatticePanelParamProc("set_structNum",structureNum,"","")
	return "#LatticePanel"
End
//
Function LatticePanelPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	Variable structureNum = str2num(StringFromList(1, popStr,":"))
//	String subWin = "microPanel#LatticePanel"
	String subWin = GetUserData("","","LatticePanelName")
	if (numtype(structureNum))		// other, structure number is manually supplied
		SetVariable set_structNum,noedit=0	,frame=1,fstyle=0,win=$subWin	// enable
	else	
		SetVariable set_structNum,noedit=1	,frame=0,fstyle=2,win=$subWin	// disable structure number
		NVAR num = root:Packages:Lattices:PanelValues:structureNum
		num = structureNum
		NVAR dirty=root:Packages:Lattices:PanelValues:dirty
		dirty = 1
	endif
	LatticePanelParamProc("set_structNum",structureNum,"","")
End
//
Function LatticePanelParamProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	String subWin = GetUserData("","","LatticePanelName")
	NVAR a=root:Packages:Lattices:PanelValues:a
	NVAR b=root:Packages:Lattices:PanelValues:b
	NVAR c=root:Packages:Lattices:PanelValues:c
	NVAR alpha=root:Packages:Lattices:PanelValues:alpha
	NVAR beta=root:Packages:Lattices:PanelValues:beta
	NVAR gam=root:Packages:Lattices:PanelValues:gam
	NVAR alphaT=root:Packages:Lattices:PanelValues:alphaT
	NVAR structureNum=root:Packages:Lattices:PanelValues:structureNum
	NVAR dirty=root:Packages:Lattices:PanelValues:dirty
	if (stringmatch(ctrlName,"set_structNum"))
		if (!(1<=varNum && varNum<=230))
			return 1
		endif
		dirty = dirty || (varNum!=str2num(GetUserData(subWin,"set_structNum","oldValue")))
		SetVariable set_structNum, userdata(oldValue)=num2istr(varNum),win=$subWin
		Variable i
		for (i=ItemsInList(PresetLatticeStructures)-1;i>=0;i-=1)
			if (varNum==str2num(StringFromList(1,StringFromList(i,PresetLatticeStructures),":")))
				break
			endif
		endfor
		if (i>=0)		// if varNum is in the list, then change back to the popup menu
			PopupMenu popLatticeStructure,mode=(i+1), win=$subWin
			SetVariable set_structNum,noedit=1	,frame=0,fstyle=2,win=$subWin						// disable structure number
		else
			PopupMenu popLatticeStructure,mode=ItemsInList(PresetLatticeStructures), win=$subWin	// set to 'other'
		endif
		String titleStr="\\JC"
		if (varNum>=195)												// Cubic, a
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_b_nm,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_c_nm,noedit=1,frame=0,win=$subWin
			SetVariable set_alpha,noedit=1,frame=0,win=$subWin
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			SetVariable set_gamma,noedit=1,frame=0,win=$subWin
			b=a  ;  c=a  ;  alpha=90 ; beta=90 ; gam=90
			titleStr += "Cubic"
		elseif (varNum>=168)											// Hexagonal, a, c
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_c_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_b_nm,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_alpha,noedit=1,frame=0,win=$subWin
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			SetVariable set_gamma,noedit=1,frame=0,win=$subWin
			b=a ; alpha=90 ; beta=90 ; gam=120
			titleStr += "Hexagonal"
		elseif (varNum>=143)											// Trigonal, a, alpha
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_alpha,noedit=0,frame=1,win=$subWin
			SetVariable set_b_nm,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_c_nm,noedit=1,frame=0,win=$subWin
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			SetVariable set_gamma,noedit=1,frame=0,win=$subWin
			b=a  ;  c=a  ;  beta=90 ; gam=90
			titleStr += "Trigonal"
		elseif (varNum>=75)												// Tetragonal, a, c
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_c_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_b_nm,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_alpha,noedit=1,frame=0,win=$subWin
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			SetVariable set_gamma,noedit=1,frame=0,win=$subWin
			b=a ; alpha=90 ; beta=90 ; gam=90
			titleStr += "Tetragonal"
		elseif (varNum>=16)												// Orthorhombic, a, b, c
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_b_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_c_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_alpha,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			SetVariable set_gamma,noedit=1,frame=0,win=$subWin
			alpha=90 ; beta=90 ; gam=90
			titleStr += "Orthorhombic"
		elseif (varNum>=3)												// Monoclinic, a, b, c, gamma
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable
			SetVariable set_b_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_c_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_gamma,noedit=0,frame=1,win=$subWin
			SetVariable set_alpha,noedit=1,frame=0,win=$subWin	// disable
			SetVariable set_beta,noedit=1,frame=0,win=$subWin
			alpha=90 ; beta=90
			titleStr += "Monoclinic"
		else																// Triclinic, a, b, c, alpha, beta, gamma
			SetVariable set_a_nm,noedit=0,frame=1,win=$subWin	// enable all
			SetVariable set_b_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_c_nm,noedit=0,frame=1,win=$subWin
			SetVariable set_alpha,noedit=0,frame=1,win=$subWin
			SetVariable set_beta,noedit=0,frame=1,win=$subWin
			SetVariable set_gamma,noedit=0,frame=1,win=$subWin
			titleStr += "Triclinic"
		endif
		titleStr += "   \\F'Courier'"+getSymString(varNum)
		titleStr = minus2bar(titleStr)		// change all minuses to a bar over following character
		TitleBox structureTitle,title=titleStr,win=$subWin

	elseif (!(stringmatch(ctrlName,"h_LatticeVar")||stringmatch(ctrlName,"k_LatticeVar")||stringmatch(ctrlName,"L_LatticeVar")))
		dirty = 1
		//	Cubic			[195,230]	//	a
		//	Hexagonal		[168,194]	//	a,c
		//	Trigonal		[143,167]	//	a,alpha
		//	Tetragonal		[75,142]		//	a,c
		//	Orthorhombic	[16,74]		//	a,b,c
		//	Monoclinic		[3,15]			//	a,b,c,gamma
		//	Triclinic		[1,2]			//	a,b,c,alpha,beta,gamma
		if (structureNum>=195)				// Cubic, a
			b=a  ;  c=a
		elseif (structureNum>=168)		// Hexagonal, a, c
			b=a
		elseif (structureNum>=143)		// Trigonal, a, alpha
			b=a  ;  c=a  ;  beta=alpha ; gam=alpha
		elseif (structureNum>=75)			// Tetragonal, a, c
			b=a
		endif
	endif

	NVAR h=root:Packages:Lattices:PanelValues:h, k=root:Packages:Lattices:PanelValues:k, l=root:Packages:Lattices:PanelValues:l
	NVAR dspace_nm=root:Packages:Lattices:PanelValues:dspace_nm
	NVAR Fr=root:Packages:Lattices:PanelValues:Fr, Fi=root:Packages:Lattices:PanelValues:Fi

	dspace_nm = get_dhkl(h,k,l)
	Variable/C Fc = getFstruct(structureNum,h,k,l)
	Fr = real(Fc)
	Fi = imag(Fc)

	// set buttons to enable/disable, based upon 'dirty'
	if (dirty)
		Button buttonLatticeSave disable=0,fColor=(57346,65535,49151),title="Save  (Needed)",win=$subWin
		Button buttonLatticeRevert disable=0,fColor=(65535,65534,49151),win=$subWin
	else
		Button buttonLatticeSave disable=2,fColor=(0,0,0),title="Saved",win=$subWin
		Button buttonLatticeRevert disable=2,fColor=(0,0,0),win=$subWin
	endif
	return 0
End
//
Function LatticePanelButtonProc(ctrlName) : ButtonControl
	String ctrlName

	SVAR desc=root:Packages:Lattices:PanelValues:desc
	NVAR structureNum=root:Packages:Lattices:PanelValues:structureNum
	NVAR a=root:Packages:Lattices:PanelValues:a
	NVAR b=root:Packages:Lattices:PanelValues:b
	NVAR c=root:Packages:Lattices:PanelValues:c
	NVAR alpha=root:Packages:Lattices:PanelValues:alpha
	NVAR beta=root:Packages:Lattices:PanelValues:beta
	NVAR gam=root:Packages:Lattices:PanelValues:gam
	NVAR alphaT=root:Packages:Lattices:PanelValues:alphaT
	NVAR dirty = root:Packages:Lattices:PanelValues:dirty
	STRUCT crystalLattice xtal
	if (stringmatch(ctrlName,"buttonLatticeRevert"))		// close window, do not save numbers
		FillLatticeStructDefault(xtal)						//fill the lattice structure with default values
		a=xtal.a  ;  b=xtal.b  ;  c=xtal.c
		alpha=xtal.alpha  ;  beta=xtal.beta  ;  gam=xtal.gam
		structureNum=xtal.structNum
		alphaT = xtal.alphaT
		desc = xtal.desc
		dirty = 0
		SetVariable set_structNum, userdata(oldValue)=num2istr(structureNum),win=$ GetUserData("","","LatticePanelName")
		LatticePanelParamProc("set_structNum",structureNum,"","")
	elseif (stringmatch(ctrlName,"buttonLatticeSave") && dirty)
		xtal.a = a  ;  xtal.b = b  ;  xtal.c = c
		xtal.alpha = alpha  ;  xtal.beta = beta  ;  xtal.gam = gam
		xtal.structNum = structureNum
		xtal.alphaT = alphaT
		xtal.desc = desc
		ForceLatticeToStructure(xtal)
		if (exists(":xtalStructStr")==2)					// use the global in this data folder
			SVAR xtalStructStr=:xtalStructStr
		elseif (exists("root:Packages:Lattices:xtalStructStr")==2)	// use the global in the Packages
			SVAR xtalStructStr=root:Packages:Lattices:xtalStructStr
		else
			String/G xtalStructStr							// no global, make one in this data folder
		endif
		StructPut/S/B=2 xtal, xtalStructStr				// and write them to the global
		String sym = getSymString(xtal.structNum)		// symmetry symbol
		printf "changed lattice to  '%s'  #%d   %s     %.9gnm, %.9gnm, %.9gnm,   %g, %g, %g\r",desc,xtal.structNum,sym,xtal.a,xtal.b,xtal.c,xtal.alpha,xtal.beta,xtal.gam
		SavePackagePreferences/FLSH=1 "LatticeSym","LatticeSymPrefs",0,xtal
		dirty = 0
		LatticePanelParamProc("set_structNum",structureNum,"","")
	elseif (stringmatch(ctrlName,"buttonLatticeFromFile"))
		if (ReadLatticeFromKeyFile("","",xtal))
			DoAlert 0,"nothing read"
		endif
		a=xtal.a  ;  b=xtal.b  ;  c=xtal.c
		alpha=xtal.alpha  ;  beta=xtal.beta  ;  gam=xtal.gam
		structureNum=xtal.structNum
		desc=xtal.desc
		alphaT=xtal.alphaT
		dirty = 1
		LatticePanelParamProc("set_structNum",structureNum,"","")
	elseif (stringmatch(ctrlName,"buttonPrintLattice"))	// print shown lattice parameters to the history
		xtal.a = a  ;  xtal.b = b  ;  xtal.c = c
		xtal.alpha = alpha  ;  xtal.beta = beta  ;  xtal.gam = gam
		xtal.structNum = structureNum
		xtal.desc = desc
		xtal.alphaT = alphaT
		ForceLatticeToStructure(xtal)
		print_crystalLattice(xtal)
	endif
	return 0
End

//	End of lattice set panel
// =========================================================================
// =========================================================================


// =========================================================================
// =========================================================================
//	Start of reading/writing lattice paremeters to a file

Function ReadLatticeFromKeyFile(fileName,path,xtal)	// read in the geometry structure from a keyword file
	String fileName										// full path name to the file
	String path											// name of an Igor path to use
	STRUCT crystalLattice &xtal							// the structure to fill from the file

	Variable tempPath = 0
	if (strlen(path)<1)
		path = "userMaterialsPath"
		PathInfo userMaterialsPath
		if (V_flag)
		else
			PathInfo Igor
			NewPath/Z/Q userMaterialsPath,S_path+"User Procedures:materials:"
			tempPath = !V_flag
			if (V_flag)
				path = "home"							// no materials folder, just look in home
			endif
		endif
	endif
	String list = keyStrFromFile(fileName,"LatticeFile",path)// read in all of the tagged values into a keyword list
	if (strlen(list)<1)
		return 1
	endif
	if (tempPath)
		KillPath/Z $path
	endif

	Variable a,b,c,alpha,beta,gam, structureNum, alphaT
	sscanf StringByKey("latticeParameters",list,"="), "{ %g, %g, %g, %g, %g, %g }"  , a,b,c,alpha,beta,gam
	if (V_flag!=6)
		return 1
	endif
	structureNum = str2num(StringByKey("latticeStructure",list,"="))
	if (numtype(structureNum))
		return 1
	endif
	String unit = StringByKey("lengthUnit",list,"=")
	if (stringmatch(unit,"Ang*"))
		a*= 10  ;  b*= 10  ;  c*= 10
	endif
	alphaT = str2num(StringByKey("latticeAlphaT",list,"="))

	xtal.a = a  ;  xtal.b = b  ;  xtal.c = c
	xtal.alpha = alpha  ;  xtal.beta = beta  ;  xtal.gam = gam
	xtal.structNum = structureNum
	xtal.desc = StringByKey("latticeDesc",list,"=")
	xtal.alphaT = !(alphaT>0) ? 0 : alphaT
	ForceLatticeToStructure(xtal)
	printf "Loaded lattice information from '%s'     using units of '%s'\r",StringByKey("keyStrFileName",list,"="),unit
	return 0
End
Static Function/S keyStrFromFile(fname,ftype,path)// read in all of the tagged values from a file into a keyword list
	String fname										// full path name to file with tagged geometry values
	String ftype										// the required file identifier, included as a tag (ftype is optional)
	String path										// name of Igor path


	Variable refNum
//	Open/M="file with a keyword list"/P=$path/R/Z=2 refNum as fname
//	if (strlen(S_fileName)<1 || !refNum)
//		return ""
//	endif
	GetFileFolderInfo/D/P=$path/Q/Z fname
	if (!V_isFile)
//		Open/D/M="file with a lattice keyword list"/P=$path/R refNum as fname
		Open/D/M="file with a lattice keyword list"/P=$path/R/T=".xtl" refNum as fname
	endif
	if (strlen(S_fileName)<1)
		return ""
	else
		Open/P=$path/R/Z=1 refNum as S_fileName
	endif
	if (strlen(S_fileName)<1 || !refNum)
		return ""
	endif
// the above 12 lines should not be necessary, the /Z=2 switch does not work right

	String line
	if (strlen(ftype))								// an ftype is present, so check the file
		FReadLine refNum, line
		line[strlen(line)-1,Inf] = " "				// change the trailing term char to a space
		ftype = "$"+ftype
		if (strsearch(line,ftype,0) || char2num(line[strlen(ftype)])>32)
			printf "The file must start with '%s', but the first line is '%s'\r",ftype,line
			Close refNum
			return ""
		endif
	endif
	String tagName,value,list = ReplaceStringByKey("keyStrFileName","",S_fileName,"=")
	Variable i,dollar = char2num("$")
	do
		FReadLine refNum, line
		if (char2num(line)==dollar)				// if it starts with a $, probable tag so process it
			i = strsearch(line,"//",0)				// strip off comments
			if (i>=0)
				line = line[0,i-1]
			endif
			for (i=0;char2num(line[i+1])>32;i+=1)// find end of tag, it ends with a space or lower
			endfor
			tagName = line[1,i]
			if (strlen(tagName)<1)
				continue
			endif

			// check if tag is already in list, only take the first instance of a tag, not the last
			if (keyInList(tagName,list,"=",""))			// check if this key is already in list
				continue
			endif

			for (i=i+1;char2num(line[i])<=32;i+=1)	// find first non-white space
			endfor
			value = line[i,Inf]							// value associated with tagName

			for (i=strlen(value)-1;char2num(value[i])<=32 && i>0;i-=1)	// strip off trailing whitespace
			endfor
			value = ReplaceString(";",value[0,i],":")
			list = ReplaceStringByKey(tagName,list,value,"=")
		endif
	while (strlen(line))									// go until EOF
	Close refNum
	return list
End

//	End of of reading/writing lattice paremeters to a file
// =========================================================================
// =========================================================================


// =========================================================================
// =========================================================================
//	Start of crystal symmetry stuff

Static Constant TRICLINIC=0,MONOCLINIC=1,ORTHORHOMBIC=2,TETRAGONAL=3,TRIGONAL=4,HEXAGONAL=5,CUBIC=6
Static Constant P_CENTER=0,F_CENTER=1,B_CENTER=2,RHOMBOHEDRAL=3,C_CENTER=4,A_CENTER=5
Static Constant FCC=225,BCC=229,DIA=227,SIMPLE_CUBIC=221,SAPPHIRE=167	// generic structure numbers


Function/C getFstruct(structNum,h,k,l)
	Variable structNum				// structure number
	Variable h,k,l
	if (!(structNum>=1 || structNum<=230) || numtype(h+k+l))
		structNum = (structNum>=1 || structNum<=230) ? structNum : 225
		h = numtype(h) ? 0 : h
		k = numtype(k) ? 0 : k
		l = numtype(l) ? 2 : l
		Prompt structNum,"structure number (from International Tables) range [1,230]"
		Prompt h,"H"
		Prompt k,"K"
		Prompt l,"L"
		DoPrompt "structure & (hkl)",structNum,h,k,l
		if (V_flag)
			return cmplx(NaN,NaN)
		endif
		if (!(structNum>=1 || structNum<=230) || numtype(h+k+l))
			return cmplx(NaN,NaN)
		endif
	endif
	Variable/C Fc = Fstruct(structNum,h,k,l)
	Variable Freal=real(Fc),Fimag=imag(Fc)
	String sym = getSymString(structNum)	// symmetry symbol
	if (ItemsInList(GetRTStackInfo(0))<2)
		printf "F(#%d, %s, %g,%g,%g) = %g %s i%g,      |F|^2 =  %g\r",structNum,sym,h,k,l,Freal,SelectString(Fimag,"-","+","+"),abs(Fimag),magsqr(Fc)
	endif
	return Fc
End


Function/C Fstruct(structNum,h,k,l)
	Variable structNum				// structure number
	Variable h,k,l
	if (!(structNum>=1 || structNum<=230) || numtype(h+k+l))
		return cmplx(NaN,NaN)	// bad inputs
	endif

	Variable Freal=0,Fimag=0

	String sym					// symmetry symbol
	Variable N						// number of vectors in xyz[][3], note the (0,0,0) is left out
	Variable latticeCentering		// lattice centering, ie face centered, body centered, ...
	Variable  arg
	Variable j
	Variable  Fr,Fi				// primarily needed for hexagonl which has the 2 atoms per cell
	Variable  rr,ii					// temporarly real and imag parts
	Variable/C zero=cmplx(0,0)

	if (exists("root:Packages:Lattices:atomxyz")!=1)
		InitLatticeSymPackage()
	endif
	Wave xyz = root:Packages:Lattices:atomxyz	// holds atom positions (at most 8 atoms)
	N = symmetryInfo(structNum,sym,xyz)	// get sym[] and xyz[N][3]

	latticeCentering = strsearch("PFIRCA",sym[0,0],0)	// takes on number [0,5]
	//	P_CENTER	F_CENTER	B_CENTER	RHOMBOHEDRAL	C_CENTER	A_CENTER
	switch (latticeCentering)
		case F_CENTER:
			if (!ALLOW_FC(h,k,l))
				return zero
			endif
			break
		case B_CENTER:
			if (!ALLOW_BC(h,k,l))
				return zero
			endif
			break
		case C_CENTER:
			if (!ALLOW_CC(h,k,l))
				return zero
			endif
			break
		case A_CENTER:
			if (!ALLOW_AC(h,k,l))
				return zero
			endif
			break
		case RHOMBOHEDRAL:
			if (!ALLOW_RHOM_HEX(h,k,l))
				return zero
			endif
	endswitch

	Fr = 1 ; Fi = 0								// correct for everything except hexagonl
	if (latticeSystem(structNum)==HEXAGONAL)
		if (!ALLOW_HEXAGONAL(h,k,l)) 
			return zero
		endif
		arg = 2*PI*((h+2*k)/3 + l*0.5)		// heagnal has atoms at (0,0,0) and (1/3, 2/3, 1/2)
		Fr = 1. + cos(arg)
		Fi = sin(arg)
		//  for hexagonal:
		//	h+2k=3n,		l=even;		F = 4*f			1
		//	h+2k=3n1,	l=odd;		F = sqrt(3)*f   sqrt(3)/4
		//	h+2k=3n1,	l=even;		F = f			1/4
		//	h+2k=3n,		l=odd; 		F = 0			0
	endif

	for (j=0;j<N;j+=1)
		arg = 2*PI*(h*xyz[j][0]+k*xyz[j][1]+l*xyz[j][2])
		Freal += cos(arg)						// 	SUM{ exp[ 2PI*i * (hx+ky+lz) ] }
		Fimag += sin(arg)
	endfor

	rr = Freal;   ii = Fimag						// (Freal,Fimag) *= (Fr,Fi)
	Freal = rr*Fr - ii*Fi
	Fimag = rr*Fi + ii*Fr

	Freal /= N
	Fimag /= N
	Freal = abs(Freal)<1e-8 ? 0 : Freal
	Fimag = abs(Fimag)<1e-8 ? 0 : Fimag
	return cmplx(Freal,Fimag)
End



Function allowedHKL(h,k,l,structNum)
	Variable h,k,l
	Variable structNum					// number denoting structure, see "#define" for FCC, BCC, etc.

	String sym							// symmetry symbol
	Variable N								// number of vectors in xyz[][3], note the (0,0,0) is left out
	Variable latticeCentering				//lattice centering, ie face centered, body centered, ...
	Variable j
	Variable Freal=0, Fimag=0

	if (exists("root:Packages:Lattices:atomxyz")!=1)
		InitLatticeSymPackage()
	endif
	Wave xyz = root:Packages:Lattices:atomxyz		// holds atom positions (at most 8 atoms)
	N = symmetryInfo(structNum,sym,xyz)		// get sym[] and xyz[N][3]

	latticeCentering = strsearch("PFIRCA",sym[0,0],0)	// takes on number [0,5]
	//	P_CENTER	F_CENTER	B_CENTER	RHOMBOHEDRAL	C_CENTER	A_CENTER

	switch (latticeCentering)
		case F_CENTER:
			if (!ALLOW_FC(h,k,l))
				return 0
			endif
			break
		case B_CENTER:
			if (!ALLOW_BC(h,k,l))
				return 0
			endif
			break
		case C_CENTER:
			if (!ALLOW_CC(h,k,l))
				return 0
			endif
			break
		case A_CENTER:
			if (!ALLOW_AC(h,k,l))
				return 0
			endif
			break
		case RHOMBOHEDRAL:
			if (!ALLOW_RHOM_HEX(h,k,l))
				return 0
			endif
	endswitch

	if (latticeSystem(structNum)==HEXAGONAL)
		if (!ALLOW_HEXAGONAL(h,k,l))
			return 0
		endif
	endif

	// made it past the first part, next check the atom positions
	Variable arg
	for (j=0;j<N;j+=1)
		arg = 2*PI*(h*xyz[j][0]+k*xyz[j][1]+l*xyz[j][2])
		Freal += cos(arg)					// 	SUM{ exp[ 2PI*i * (hx+ky+lz) ] }
		Fimag += sin(arg)
	endfor
	return ( (Freal*Freal+Fimag*Fimag)/(N*N) > 0.01 )
End



Function/S getSymString(structNum)
	Variable structNum		//structure number
	String sym
	symmetryInfo(structNum,sym,$"")
	return sym
End
//
//
// returns info about the symmetry of a structure
// sym holds the symmetry info on return, and xyz holds the atom positions.  It returns the number of atom positions put in xyz
// which is always at least 1.  If you call with a bad wave ref for xyz, then it only returns the sym string, and the returned value is 0
Static Function symmetryInfo(structNum,sym,xyz)
	Variable structNum					//structure number
	String &sym							// symmetry symbol to be set
	Wave xyz								// needs to be dimensioned /N=(8,3),  there are at most 8 sites for a structure
	if (structNum<1 || structNum>230)
		sym = ""							// invalid structNum
		return 0
	endif

	// there are 230 items in this array
	String symms="P1;P-1;P2:b;P21:b;C2:b1;Pm:b;Pc:b1;"
	symms += "Cm:b1;Cc:b1;P2/m:b;P21/m:b;C2/m:b1;P2/c:b1;"
	symms += "P21/c:b1;C2/c:b1;P222;P2221;P21212;P212121;"
	symms += "C2221;C222;F222;I222;I212121;Pmm2;Pmc21;"
	symms += "Pcc2;Pma2;Pca21;Pnc2;Pmn21;Pba2;Pna21;"
	symms += "Pnn2;Cmm2;Cmc21;Ccc2;Amm2;Abm2;Ama2;Aba2;"
	symms += "Fmm2;Fdd2;Imm2;Iba2;Ima2;Pmmm;Pnnn:1;Pccm;"
	symms += "Pban:1;Pmma;Pnna;Pmna;Pcca;Pbam;Pccn;Pbcm;"
	symms += "Pnnm;Pmmn:1;Pbcn;Pbca;Pnma;Cmcm;Cmca;Cmmm;"
	symms += "Cccm;Cmma;Ccca:1;Fmmm;Fddd:1;Immm;Ibam;Ibca;"
	symms += "Imma;P4;P41;P42;P43;I4;I41;P-4;I-4;"
	symms += "P4/m;P42/m;P4/n:1;P42/n:1;I4/m;I41/a:1;P422;"
	symms += "P4212;P4122;P41212;P4222;P42212;P4322;P43212;"
	symms += "I422;I4122;P4mm;P4bm;P42cm;P42nm;P4cc;"
	symms += "P4nc;P42mc;P42bc;I4mm;I4cm;I41md;I41cd;"
	symms += "P-42m;P-42c;P-421m;P-421c;P-4m2;P-4c2;P-4b2;"
	symms += "P-4n2;I-4m2;I-4c2;I-42m;I-42d;P4/mmm;P4/mcc;"
	symms += "P4/nbm:1;P4/nnc:1;P4/mbm;P4/mnc;P4/nmm:1;P4/ncc:1;"
	symms += "P42/mmc;P42/mcm;P42/nbc:1;P42/nnm:1;P42/mbc;P42/mnm;"
	symms += "P42/nmc:1;P42/ncm:1;I4/mmm;I4/mcm;I41/amd:1;"
	symms += "I41/acd:1;P3;P31;P32;R3:H;P-3;R-3:H;P312;"
	symms += "P321;P3112;P3121;P3212;P3221;R32:H;P3m1;P31m;"
	symms += "P3c1;P31c;R3m:H;R3c:H;P-31m;P-31c;P-3m1;P-3c1;"
	symms += "R-3m:H;R-3c:H;P6;P61;P65;P62;P64;P63;P-6;"
	symms += "P6/m;P63/m;P622;P6122;P6522;P6222;P6422;P6322;"
	symms += "P6mm;P6cc;P63cm;P63mc;P-6m2;P-6c2;P-62m;P-62c;"
	symms += "P6/mmm;P6/mcc;P63/mcm;P63/mmc;P23;F23;I23;P213;"
	symms += "I213;Pm-3;Pn-3:1;Fm-3;Fd-3:1;Im-3;Pa-3;Ia-3;"
	symms += "P432;P4232;F432;F4132;I432;P4332;P4132;I4132;"
	symms += "P-43m;F-43m;I-43m;P-43n;F-43c;I-43d;Pm-3m;"
	symms += "Pn-3n:1;Pm-3n;Pn-3m:1;Fm-3m;Fm-3c;Fd-3m:1;Fd-3c:1;"
	symms += "Im-3m;Ia-3d;"
	sym = StringFromList(structNum-1,symms)	// set the symmetry symbol
	if (!WaveExists(xyz))							// a null wave was passed, so only return the symmetry symbol
		return 0
	endif

	String xyzList=""									// there are 230 items in this array
	// the numbers are the reciprocal, so a 4 is really 0.25, and -6 is really -0.1666667
	xyzList += ";;;0 2 0;;;0 0 2;;0 0 2;;0 2 0;;0 0 2;0 2 2;"
	xyzList += "0 0 2;;0 0 2;2 2 0;2 0 2,2 2 0,0 2 2;0 0 2;;;;0 2 0,0 0 2,2 0 0;"
	xyzList += ";0 0 2;0 0 2;2 0 0;0 0 2,2 0 2,2 0 0;0 2 2;2 0 2;2 2 0;0 0 2,2 2 2,2 2 0;"
	xyzList += "2 2 2;;0 0 2;0 0 2;;0 0 2;2 0 0;2 0 2;;4 4 4;;0 0 2;2 0 0;;"
	xyzList += "2 2 2;0 0 2;2 2 0;2 0 0;2 0 0,0 2 2,2 2 2;2 0 2;2 0 0,2 0 2,0 0 2;2 2 0;"
	xyzList += "2 2 0,2 0 2,0 2 2;0 0 2,0 2 0,0 2 2;2 2 2;2 2 0;2 2 2,2 2 0,0 0 2;2 0 2,2 2 0,0 2 2;"
	xyzList += "2 0 2,2 2 2,0 2 0;0 0 2;0 2 2;;0 0 2;0 2 0;0 2 2;;4 4 4;;0 0 2;"
	xyzList += "0 2 0,0 0 2,2 0 0;0 2 0;;0 0 4,0 0 2,0 0 -4;0 0 2;0 0 -4,0 0 2,0 0 4;;"
	xyzList += "0 2 4;;;;0 0 2;2 2 0;2 2 2;;0 2 4;;2 2 0;0 0 4,0 0 2,0 0 -4;"
	xyzList += "2 2 4,0 0 2,2 2 -4;0 0 2;2 2 2;0 0 -4,0 0 2,0 0 4;2 2 -4,0 0 2,2 2 4;;"
	xyzList += "0 2 4;;2 2 0;0 0 2;2 2 2;0 0 2;2 2 2;0 0 2;0 0 2,2 2 0,2 2 2;;"
	xyzList += "0 0 2;0 2 4;0 2 4,0 0 2,2 0 4;;0 0 2;2 2 0;2 2 2;;0 0 2;2 2 0;"
	xyzList += "2 2 2;;0 0 2;;0 2 4;;0 0 2;2 2 0;2 2 2;2 2 0;2 2 2;2 2 0;"
	xyzList += "2 2 0,2 2 2,0 0 2;0 0 2;0 0 2;2 2 2,0 0 2,2 2 0;2 2 2;0 0 2,2 2 0,2 2 2;"
	xyzList += "2 2 2;2 2 2;2 2 2,2 2 0,0 0 2;;0 0 2;0 2 4;0 2 4,2 0 4,0 0 2;;"
	xyzList += "0 0 3,0 0 -3;0 0 -3,0 0 3;;;;;;0 0 3,0 0 -3;0 0 3,0 0 -3;"
	xyzList += "0 0 -3,0 0 3;0 0 -3,0 0 3;;;;0 0 2;0 0 2;;0 0 2;;0 0 2;;"
	xyzList += "0 0 2;;0 0 2;;0 0 6,0 0 3,0 0 2,0 0 -3,0 0 -6;0 0 -6,0 0 -3,0 0 2,0 0 3,0 0 6;"
	xyzList += "0 0 3,0 0 -3;0 0 -3,0 0 3;0 0 2;;;0 0 2;;0 0 6,0 0 3,0 0 2,0 0 -3,0 0 -6;"
	xyzList += "0 0 -6,0 0 -3,0 0 2,0 0 3,0 0 6;0 0 3,0 0 -3;0 0 -3,0 0 3;0 0 2;;0 0 2;"
	xyzList += "0 0 2;0 0 2;;0 0 2;;0 0 2;;0 0 2;0 0 2;0 0 2;;;;2 0 2,2 2 0,0 2 2;"
	xyzList += "0 2 0,0 0 2,2 0 0;;2 2 2;;4 4 4;;2 0 2,2 2 0,0 2 2;0 2 0,0 0 2,2 0 0;;"
	xyzList += "2 2 2;;4 4 4;;-4 4 -4,2 0 2,-4 -4 4,2 2 0,4 -4 -4,0 2 2,4 4 4;"
	xyzList += "4 -4 4,2 0 2,4 4 -4,2 2 0,-4 4 4,0 2 2,-4 -4 -4;4 -4 4,0 2 0,4 4 -4,0 0 2,-4 4 4,2 0 0,4 4 4;"
	xyzList += ";;;2 2 2;0 0 2;4 -4 4,0 2 0,4 4 -4,0 0 2,-4 4 4,2 0 0,4 4 4;;2 2 2;2 2 2;"
	xyzList += "2 2 2;;0 0 2;4 4 4;4 4 -4,4 4 4,0 0 2;;4 -4 4,0 2 0,4 4 -4,0 0 2,-4 4 4,2 0 0,4 4 4;"
	Variable xx,yy,zz, N												// number atom points in xyzStr plus one (for the 000)
	String svec,	list = StringFromLIst(structNum-1,xyzList)	// list of vectors for this symmetry
	xyz = NaN
	xyz[0][] = 0														// first site is always (0,0,0)
	for (N=1,svec=StringFromLIst(0,list,",");char2num(svec)>0 && N<8;N+=1,svec=StringFromLIst(N-1,list,","))
		xx = str2num(svec)
		yy = str2num(StringFromLIst(1,svec," "))
		zz = str2num(StringFromLIst(2,svec," "))
		xyz[N][0] = xx ? 1/xx : xx									// values were stored as reciprocals
		xyz[N][1] = yy ? 1/yy : yy
		xyz[N][2] = zz ? 1/zz : zz
	endfor
	return N
End



Static Function latticeSystem(structNum)
	Variable structNum				// structure number
	if (structNum>230)
		return -1					  	 // invalid
	elseif (structNum>=195)
		return CUBIC
	elseif (structNum>=168)
		return HEXAGONAL
	elseif (structNum>=143)
		return TRIGONAL				// Trigonal, (using the hexagonal cell axes)
	elseif (structNum>=75)
		return TETRAGONAL
	elseif (structNum>=16)
		return ORTHORHOMBIC
	elseif (structNum>=3)
		return MONOCLINIC
	elseif  (structNum>0)
		return TRICLINIC
	endif
	return -1							// invalid
End



//	#define ALLOW_FC(H,K,L) (!(((H)+(K))%2) && !(((K)+(L))%2))		// H,K,L all even or all odd
//	#define ALLOW_BC(H,K,L) (!(((H)+(K)+(L))%2))					// !mod(round(h+k+l),2)
//	#define ALLOW_CC(H,K,L) (!((H)+(K))%2)							// !mod(round(h+k),2)
//	#define ALLOW_AC(H,K,L) (!((K)+(L))%2)							// !mod(round(k+l),2)
//	#define ALLOW_RHOM_HEX(H,K,L) (((-(H)+(K)+(L))%3)==0 || (((H)-(K)+(L))%3)==0)   // allowed are -H+K+L=3n or H-K+L=3n
//	#define ALLOW_HEXAGONAL(H,K,L) ((((H)+2*(K))%3) || !((L)%2))   // forbidden are: H+2K=3N with L odd
//
Static Function ALLOW_FC(h,k,l)				// for face-centered, hkl must be all even or all odd
	Variable h,k,l
	return !mod(h+k,2) && !mod(k+l,2)
End
//
Static Function ALLOW_BC(h,k,l)				// for body-centered, !mod(round(h+k+l),2)
	Variable h,k,l
	return !mod(round(h+k+l),2)
End
//
Static Function ALLOW_CC(h,k,l)				// for C-centered, !mod(round(h+k),2)
	Variable h,k,l
	return !mod(round(h+k),2)
End
//
Static Function ALLOW_AC(h,k,l)				// for A-centered, !mod(round(k+l),2)
	Variable h,k,l
	return !mod(round(k+l),2)
End
//
Static Function ALLOW_RHOM_HEX(h,k,l)		// for rhombohedral hexagonal, allowed are -H+K+L=3n or H-K+L=3n
	Variable h,k,l
	return !mod(-h+k+l,3) || !mod(h-k+l,3)
End
//
Static Function ALLOW_HEXAGONAL(h,k,l)		// for hexagonal, // forbidden are: H+2K=3N with L odd
	Variable h,k,l
	return mod(h+2*k,3) || !mod(l,2)
End

//	End of crystal symmetry stuff
// =========================================================================
// =========================================================================

// =========================================================================
// =========================================================================
//	Start of some utility routines

// changes hkl[3] to the lowest order hkl, ignores whether a reflection is allowed, just removes common factors
Function lowestOrderHKL(h,k,l)
	Variable &h,&k,&l							// these hkl are returned with all common factors removed
	Variable maxDiv								// max possible divisor
	Variable i
	maxDiv = max(abs(h),abs(k))				// the maximum divisor cannot be bigger than the smallest of hkl
	maxDiv = max(maxDiv,abs(l))
	for (i=maxDiv;i>=2;i-=1)					// check all divisorts in range [2, maxDiv]
		if (mod(h,i) || mod(k,i) || mod(l,i))	// i is not a factor of h, k, and l
			continue
		endif
		h /= i
		k /= i
		l /= i
	endfor
End


// changes hkl[3] to the lowest order allowed hkl (ie for FCC, 0,0,12 -> 002 not 001
Function lowestAllowedHKL(h,k,l,structNum)
	Variable &h,&k,&l
	Variable structNum					// number denoting structure, see "#define" for FCC, BCC, etc.

	Variable i
	Variable hh=h, kk=k, ll=l
	lowestOrderHKL(hh,kk,ll)			// remove all common factors

	for (i=1;i<16;i+=1)					// never need more than 16 to reach an allowed reflection
		h = i*hh							// try each of the multiples to reach an allowed reflection
		k = i*kk
		l = i*ll
		if (allowedHKL(h,k,l,structNum))
			return 0
		endif
	endfor
	return 0
End


Function/S hkl2str(h,k,l)				// format h,k,l into a string of acceptable minimal length
	Variable h,k,l
	if (numtype(h+k+l))
		return "(NaN,NaN,NaN)"
	endif
	String hkl
	if (k<0 || l<0)
		sprintf hkl,"%d, %d, %d",h,k,l
	elseif (abs(h)<10 && k<10 && l<10)
		sprintf hkl,"%d%d%d",h,k,l
	else
		sprintf hkl,"%d %d %d",h,k,l
	endif
	return hkl
End
//Function/S hkl2str(h,k,l)		// format h,k,l into a string of acceptable minimal length
//	Variable h,k,l
//	if (numtype(h+k+l))
//		return "(NaN,NaN,NaN)"
//	endif
//	String hkl
//	if (h<0 || k<0 || l<0)
//		sprintf hkl,"%d, %d, %d",h,k,l
//	elseif (h<10 && k<10 && l<10)
//		sprintf hkl,"%d%d%d",h,k,l
//	else
//		sprintf hkl,"%d %d %d",h,k,l
//	endif
//	return hkl
//End


Function/T minus2bar(str)		// change an Igor string that has minuses to one using a bar over following character
	String str
	String bs
	Variable i
	for (i=0; i<=9 && strsearch(str,"-",0)>=0; i+=1)
		sprintf bs, "\\[%d\\S\\M\\X%d",i,i
		str = ReplaceString("-",str,bs,0,1)
	endfor
	return str
End


Function placesOfPrecision(a)
	Variable a
	a = roundSignificant(abs(a),17)
	Variable i
	for (i=1;i<18;i+=1)
		if (abs(a-roundSignificant(a,i))/a<1e-15)
			break
		endif
	endfor
	return i
End

//	End of utility
// =========================================================================
// =========================================================================


Function InitLatticeSymPackage()				// used to initialize this package
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:Lattices
	Make/N=(8,3)/O root:Packages:Lattices:atomxyz=NaN
	String/G root:Packages:Lattices:latticeNames="Triclinic;Monoclinic;Orthorhombic;Tetragonal;Trigonal;Hexagonal;Cubic"
End