#pragma rtGlobals=1		// Use modern global access method.

Function SiLatticeConst(TempC)
	Variable TempC		// temperature in degrees C
	Variable aSi=5.43102088			// lattice constant of Si at 22.5 ()
	Variable alpha=2.56E-6			// coefficient of expansion (1/K)
	Variable dT = TempC - 22.5
	return aSi*(1+alpha*dT)
End

Function PhysicalConstantsInitPackage()
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:Constants
	Variable/G root:Packages:Constants:c=299792458				// speed of light (m/s)
	Variable/G root:Packages:Constants:eo=8.854187817			// permitivity of vacuum  (F/m)
	Variable/G root:Packages:Constants:G=6.673E-11				// Gravitational constant (m^3 1/kg 1/s)
	Variable/G root:Packages:Constants:h_Js=6.62606876E-34		// Plank constant (J s)
	Variable/G root:Packages:Constants:h_eVs=4.13566727e-15	// Plank constant (eV s)
	Variable/G root:Packages:Constants:e=1.602176462e-19		// Chargeon electron (C)
	Variable/G root:Packages:Constants:alpha=7.297352533e-3		// fine structure constant
	Variable/G root:Packages:Constants:aB=0.5291772083			// Bohr radius (m)
	Variable/G root:Packages:Constants:Rydberg=13.60569172		// Rydberg (eV) (=alpha^2*me*c/2/h * hc)
	Variable/G root:Packages:Constants:me_kg=9.10938188e-31	// mass of electron (kg)
	Variable/G root:Packages:Constants:me_keV=510.998902		// mass of electron (keV)
	Variable/G root:Packages:Constants:me_mp=5.446170232e-4	// mass ratio m(electron)/m(proton)
	Variable/G root:Packages:Constants:mp_kg=1.67262171e-27	// mass of proton (kg)
	Variable/G root:Packages:Constants:mn_kg=1.67492716e-27	// mass of neutron (kg)
	Variable/G root:Packages:Constants:mn_eV=9.3956533237e8	// mass of neutron (eV)
	Variable/G root:Packages:Constants:NA=6.02214199e23			// Avagadro's number
	Variable/G root:Packages:Constants:kB_JK=1.3806503e-23		// Boltzman constant (J/K)
	Variable/G root:Packages:Constants:kB_eVK=8.617342e-5		// Boltzman constant (eV/K)
	Variable/G root:Packages:Constants:J_eV=1.602176462e-19	// Joules/eV
	Variable/G root:Packages:Constants:atm_Pa=101325			// Number of Pa in one atm
	Variable/G root:Packages:Constants:hc=12.39841857			// h*c (keV-)
	Variable/G root:Packages:Constants:re_m=2.817940285e-15	// Thompson radius (m)
	Variable/G root:Packages:Constants:re=2.817940285e-5		// Thompson radius ()
	Variable/G root:Packages:Constants:CuKa1=1.540593226		// wavelength of Cu Kalpha(1) ()
	Variable/G root:Packages:Constants:MoKa1=0.709317155		// wavelength of Mo Kalpha(1) ()
	Variable/G root:Packages:Constants:aSi=5.43102088			// lattice constant of Si at 22.5 ()
	String/G root:Packages:Constants:aSi_T = "aSi*(1+2.56E-6*(TempC-22.5))"	// aSi(T)
//	String/G root:Packages:Constants:symbols
//	SVAR symbols=root:Packages:Constants:symbols
//	symbols ="H;He;Li;Be;B;C;N;O;F;Ne;Na;Mg;Al;Si;P;S;Cl;Ar;"
//	symbols += "K;Ca;Sc;Ti;V;Cr;Mn;Fe;Co;Ni;Cu;Zn;Ga;Ge;As;Se;Br;Kr;"
//	symbols += "Rb;Sr;Y;Zr;Nb;Mo;Tc;Ru;Rh;Pd;Ag;Cd;In;Sn;Sb;Te;I;Xe;"
//	symbols += "Cs;Ba;La;Ce;Pr;Nd;Pm;Sm;Eu;Gd;Tb;Dy;Ho;Er;Tm;Yb;Lu;"
//	symbols += "Hf;Ta;W;Re;Os;Ir;Pt;Au;Hg;Tl;Pb;Bi;Po;At;Rn;Fr;Ra;Ac;Th;Pa;U;"
//	symbols += "Np;Pu;Am;Cm;Bk;Cf;Es;Fm;Md;No;Lr;Rf;Db;Sg;Bh;Hs;Mt"
EndMacro

//			to get the Z of an element
//
//Function element2Z(symb)	// returns Z for an atomic symbol (CASE SENSITIVE)
//	String symb						// atomic symbol
//	SVAR symbols=root:Packages:Constants:symbols
//	symb[0,0] = UpperStr(symb[0,0])		// ensure first char is upper case
//	symb[1,1] = LowerStr(symb[1,1])		// and second character is lower
//	Variable iz = WhichListItem(symb,symbols)+1
//	return ((iz>0) ? iz : NaN)
//End


