#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=StereoGraphicProjection
#pragma version = 2.0
#include "LatticeSym", version>=1.3


Menu "Analysis"
	"Stereographic Projection", MakeStereo(NaN,NaN,NaN,NaN,NaN)
End


Function MakeStereo(Hz,Kz,Lz,hklmax,fntSize)
	Variable Hz,Kz,Lz			// hkl of the pole
	Variable hklmax
	Variable fntSize

	if (!(fntSize>0) || !(hklmax>0) || numtype(Hz+Kz+Lz))
		Hz = numtype(Hz) ? 3 : Hz
		Kz = numtype(Kz) ? -1 : Kz
		Lz = numtype(Lz) ? 7 : Lz
		fntSize = fntSize>0 ? fntSize : 9
		hklmax = hklmax>0 ? hklmax : 6
		String hklStr
		sprintf hklStr,"%d %d %d",Hz,Kz,Lz
		Prompt hklStr,"(hkl) of the pole"
		Prompt fntSize,"size of font for hkl"
		Prompt hklmax,"maximum hkl to accept"
		DoPrompt "hkl max & font size",hklStr,hklmax,fntSize
		if (V_flag)
			return 1
		endif
		sscanf hklStr, "%d %d %d",Hz,Kz,Lz
		printf "MakeStereo(%g,%g,%g,  %g,%g)\r",Hz,Kz,Lz,hklmax,fntSize
	endif
	if (!(fntSize>0) || !(hklmax>0) || numtype(Hz+Kz+Lz))
		return 1
	endif
	if (setLattice())
		return 1
	endif

	STRUCT crystalLattice xtal
	if (FillLatticeStructDefault(xtal))					//fill the lattice structure with current values
		DoAlert 0, "No Lattice, please set one"
		return 1
	endif

	Variable maxWaveLen=9000
	Make/N=(maxWaveLen)/O longs,lats
	Make/N=(maxWaveLen)/T/O hkls
	longs=NaN
	lats=NaN
	Variable H,K,L
	Variable N=0, lat, long
	String str
	for (L=0;L<=hklmax;L=signedAdvance(L))
		for (K=0;K<=hklmax;K=signedAdvance(K))
			for (H=0;H<=hklmax;H=signedAdvance(H))
				if (magsqr(Fstruct(xtal.structNum,h,k,l)) < 0.001)
					continue
				endif
				if (H==0 && K==0 && L==0)		// skip the (000)
					continue
				endif
				hkl2LongLat(Hz,Kz,Lz,h,k,l,long,lat,xtal)
				if (lat>=-0.01 && N<maxWaveLen)
					longs[N] = long
					lats[N] = lat
//					sprintf str,"%d %d %d",H,K,L
//					hkls[N] = str
					hkls[N] = ReplaceString(",",hkl2str(H,K,L),"")
					N += 1
				endif
			endfor
		endfor
	endfor
	Redimension/N=(N) longs,lats
	Project/M=1 longs, lats
	Wave W_XProjection=W_XProjection, W_YProjection=W_YProjection

	// remove duplicates
	Variable i, j
	for (i=N-1;i>=0;i-=1)
		for (j=0;j<i;j+=1)
			if (abs(W_XProjection[i]-W_XProjection[j])+abs(W_YProjection[i]-W_YProjection[j]) < 0.01)
				W_XProjection[i] = NaN
				W_YProjection[i] = NaN
				break
			endif
		endfor
	endfor

	Display/W=(358.5,41,848.25,518.75)/K=1		// draw the graph
	NewFreeAxis/O/L emptyLeft
	NewFreeAxis/O/B emptyBottom
	ModifyGraph gfMult=60,width={Aspect,1},tick=3,noLabel=2,standoff=0,axThick=0,freePos=0,freePos=0
	SetAxis emptyLeft -2,2
	SetAxis emptyBottom -2,2

//	sprintf str, "\Zr142#%d   %s\r\\Zr141(%d %d %d)\r90\\F'Symbol'\\F]0",xtal.structNum,getSymString(xtal.structNum),Hz,Kz,Lz
	sprintf str, "\Zr142#%d   %s\r\\Zr141(%s)\r90\\F'Symbol'\\F]0",xtal.structNum,getSymString(xtal.structNum),hkl2str(Hz,Kz,Lz)
	TextBox/N=pole/F=0/A=LT/X=0/Y=0 str
	Button zoomIn,pos={16,65},size={30,20},proc=ZoomStereoButtonProc,title="+"
	Button zoomOut,pos={16,85},size={30,20},proc=ZoomStereoButtonProc,title=""

	String list = ReplaceNumberByKey("stereoRadius","",90,"=")
	list = ReplaceNumberByKey("Hz",list,Hz,"=")
	list = ReplaceNumberByKey("Kz",list,Kz,"=")
	list = ReplaceNumberByKey("Lz",list,Lz,"=")
	SetWindow kwTopWin, userdata(stereoRadius)=list

	SetDrawLayer/K UserFront
	if (1)
		SetDrawEnv xcoord= emptyBottom,ycoord= emptyLeft,fillpat= 0
		SetDrawEnv linefgc= (43520,43520,43520)
		DrawOval -0.01,-0.01,0.01,0.01
	else
		SetDrawEnv xcoord= emptyBottom,ycoord= prel
		DrawLine 0,0.49,0,0.51				// draw a small cross at the center
		SetDrawEnv xcoord= prel,ycoord= emptyLeft
		DrawLine 0.49,0,0.51,0
	endif

	SetDrawEnv fillpat= 0
	DrawOval 0,0,1,1						// draw big circle
	for (i=0;i<N;i+=1)						// draw all of the hkl on the graph
		if (numtype(W_XProjection[i]+W_YProjection[i]))
			continue
		endif
		SetDrawEnv xcoord= emptyBottom,ycoord= emptyLeft,textxjust= 1,textyjust= 1,fsize=fntSize
		DrawText W_XProjection[i],W_YProjection[i],hkls[i]
	endfor

	KillWaves/Z xhat,yhat,zhat, hkl, W_YProjection,W_XProjection, W_cross
	KillWaves/Z lats, longs, hkls
End
//
Function signedAdvance(i)
	Variable i
	i = round(i)
	return (i<0) ? abs(i) : -(i+1)
End
//
Function ZoomStereoButtonProc(ctrlName) : ButtonControl
	String ctrlName

	String list = GetUserData("","","stereoRadius")
	Variable Hz = NumberByKey("Hz",list,"=")
	Variable Kz = NumberByKey("Kz",list,"=")
	Variable Lz = NumberByKey("Lz",list,"=")
	Variable stereoRadius = NumberByKey("stereoRadius",list,"=")
	stereoRadius = numtype(stereoRadius) ? 90 : stereoRadius

	if (stringmatch(ctrlName,"zoomIn"))
		stereoRadius -= 5
	elseif (stringmatch(ctrlName,"zoomOut"))
		stereoRadius += 5
	else
		return 1
	endif
	stereoRadius = limit(stereoRadius,5,90)
	Variable d = 2*sin(stereoRadius*PI/180) / (1+cos(stereoRadius*PI/180))
	SetAxis emptyLeft -d,d
	SetAxis emptyBottom -d,d
	String str
	STRUCT crystalLattice xtal
	if (FillLatticeStructDefault(xtal))					//fill the lattice structure with current values
		sprintf str, "\\Zr200(%s)\r%g\\F'Symbol'\\F]0",hkl2str(Hz,Kz,Lz),stereoRadius
	else
		sprintf str, "\\Zr142#%d   %s\r\\Zr141(%s)\r%g\\F'Symbol'\\F]0",xtal.structNum,getSymString(xtal.structNum),hkl2str(Hz,Kz,Lz),stereoRadius
	endif
	TextBox/C/N=pole/F=0/A=LT/X=0/Y=0 str
	list = ReplaceNumberByKey("stereoRadius",list,stereoRadius,"=")
	SetWindow kwTopWin, userdata(stereoRadius)=list
	return 0
End
//
Static Function hkl2LongLat(Hz,Kz,Lz,h,k,l,long,lat,xtal)
	Variable h,k,l
	Variable &long, &lat
	Variable Hz,Kz,Lz			// hkl of the pole
	STRUCT crystalLattice &xtal

	Make/N=3/O/D xhat,yhat,zhat, hkl
	zhat[0] = Hz*xtal.as0+Kz*xtal.bs0+Lz*xtal.cs0	// zhat =  recip x {Hz,Kz,Lz}
	zhat[1] = Hz*xtal.as1+Kz*xtal.bs1+Lz*xtal.cs1
	zhat[2] = Hz*xtal.as2+Kz*xtal.bs2+Lz*xtal.cs2
	FindPerpVector(zhat,xhat)

	Cross zhat, xhat
	Wave W_Cross=W_Cross
	yhat = W_Cross
	normalize(xhat)
	normalize(zhat)
	normalize(yhat)
	hkl[0] = h*xtal.as0+k*xtal.bs0+l*xtal.cs0		// hkl =  recip x hkl
	hkl[1] = h*xtal.as1+k*xtal.bs1+l*xtal.cs1
	hkl[2] = h*xtal.as2+k*xtal.bs2+l*xtal.cs2
	// for a non-cubic lattice, here is where the change will go
	// add here:   hkl =  recip x hkl

	normalize(hkl)
	lat = 90 - acos(min(MatrixDot(zhat,hkl),1))*180/PI
	long = atan2(MatrixDot(yhat,hkl),MatrixDot(xhat,hkl))*180/PI
	return 0
End
//
Function FindPerpVector(ref,perp)	// find a vector perpendicular to ref
	Wave ref						// reference vector, find a vec perp to this
	Wave perp					// resulting perpendicular vector, not normalized
	Make/N=3/O/D FindPerpVector_hat
	Wave hat=FindPerpVector_hat

	hat = abs(ref)
	WaveStats/M=1/Q hat
	hat = 0
	hat[V_maxLoc] = 1
	Variable dot = MatrixDot(ref,hat)
	perp = ref - dot*hat
	Cross ref, perp
	Wave W_cross=W_cross
	perp = W_cross
	KillWaves/Z FindPerpVector_hat, W_cross
	return 0
End


//Function ALLOW_All(h,k,l)				// allow all hkl, always returns 1
//	Variable h,k,l
//	return 1
//End
////
//Static Function ALLOW_FC(h,k,l)				// for face-centered, hkl must be all even or all odd
//	Variable h,k,l
//	return !mod(h+k,2) && !mod(k+l,2)
//End
////
//Static Function ALLOW_BC(h,k,l)				// for body-centered, !mod(round(h+k+l),2)
//	Variable h,k,l
//	return !mod(round(h+k+l),2)
//End
////
//Static Function ALLOW_CC(h,k,l)				// for C-centered, !mod(round(h+k),2)
//	Variable h,k,l
//	return !mod(round(h+k),2)
//End
////
//Static Function ALLOW_AC(h,k,l)				// for A-centered, !mod(round(k+l),2)
//	Variable h,k,l
//	return !mod(round(k+l),2)
//End
////
//Static Function ALLOW_RHOM_HEX(h,k,l)		// for rhombohedral hexagonal, allowed are -H+K+L=3n or H-K+L=3n
//	Variable h,k,l
//	return !mod(-h+k+l,3) || !mod(h-k+l,3)
//End
////
//Static Function ALLOW_HEXAGONAL(h,k,l)		// for hexagonal, // forbidden are: H+2K=3N with L odd
//	Variable h,k,l
//	return mod(h+2*k,3) || !mod(l,2)
//End
